mata:
mata clear

void function simulateDataset(string scalar in_gamma, string scalar in_rho, real scalar in_n){
//void function simulateDataset(real matrix in_gamma, real matrix in_rho, real in_n){
	gammaM = st_matrix(in_gamma)
	//stata(`"dis "gammaM = ""')
	//gammaM
	rho = st_matrix(in_rho)
	//stata(`"dis "rho = ""')
	//rho	
    nclass = cols(gammaM)
	//stata(`"dis "nclass = ""')
	//nclass
	nitems = rows(rho)
	//stata(`"dis "nitems = ""')
	//nitems
	n = in_n
	response = J(n, nitems, .)
	for( i = 1; i<= n; i++) {
	    //printf("i is %f\n", i)
		if(nclass > 1) {
			this_person_class = rdiscrete(1, 1, gammaM)
		}
		else {
			this_person_class = 1
		}
		//printf("the class is %f\n", this_person_class)
	    for( j = 1; j<= nitems; j++) {
			//printf("j is %f\n", j)
		    //printf("rho is %f %f %f\n", j, this_person_class, rho[j,this_person_class])
			response[i,j] = rdiscrete(1, 1, (rho[j,this_person_class],1-rho[j,this_person_class]))
			//printf("response is %f\n", response[i,j])
		}
	}
	//response[1..30,]
    responsePatterns = J(2^nitems,nitems,0)
	for( item_i = 1; item_i<= nitems; item_i++) {
		temp_a = (1 \ 2)
		temp_b = J(2^(nitems-item_i), 1, 1)
		temp_c = temp_a#temp_b
		temp_d = J(2^(item_i - 1), 1, 1)
		responsePatterns[.,item_i] = temp_d#temp_c
	}
	//responsePatterns
	Count = J(2^nitems,1,0)
	for( i = 1; i<= n; i++) {
		obv = response[i,]
		for( item_i = 1; item_i<= 2^nitems; item_i++) {
			if( responsePatterns[item_i,] == obv ) {
				Count[item_i] = Count[item_i] + 1
			}
		}
    }	
	//column join responsePatterns and responsePatterns
	response_matrix = responsePatterns , Count 
	//response_matrix
    st_addobs(rows(response_matrix))
	//varidx = st_addvar("int", ("var1","var2","var3","var4","var5","count"))
    varidx = st_addvar("int", st_tempname(cols(response_matrix)))
    st_store(.,varidx,response_matrix)
}
end

capture program drop doLcaBootstrap
program doLcaBootstrap, eclass
	version 11.0
	/* set more off		shall remain off for all commands */

	syntax,						///
		null_gammalist(numlist)			///
		null_rholist(numlist)             ///
		null_loglikelihood(real)  		///
		alt_loglikelihood(real)  		///
		simulate_samplesize(integer)    ///
		[num_bootstrap(integer 99)       ///
		null_nstarts(integer 20)     	///
		alt_nstarts(integer 20)         ///
        cores(integer 1)]        
	
	tempname nclass nitems pvalue realTestStat bootstrapTestStat gammaM rho loglikelihoodDiff bootstrapTestStat pvalue
	
	//set matsize `=`num_bootstrap'+10'	 
	local classCount: word count `null_gammalist'
	scalar `nclass'= `classCount'
	local numList : word count `null_rholist'
	scalar `nitems' = `numList'/`nclass'
	if(`nitems' * `nclass' != `numList'){
		display as error "The input gamma and rho are incompatible."
		exit 198	
	}
	if(`null_nstarts' < 10){
		display as error "A higher value for null_nstarts is needed."
		exit 198	
	}
	if(`alt_nstarts' < 10){
		display as error "A higher value for alt_nstarts is needed."
		exit 198	
	}
	
	
	//matrix define `loglikelihoodDiff' = J(`num_bootstrap',1,0)
	//matrix define `bootstrapTestStat' = J(`num_bootstrap',1,0)
	scalar `loglikelihoodDiff' = 0.0
	scalar `bootstrapTestStat' = 0.0
	matrix define `gammaM'= J( 1, `nclass', 0.0)
	matrix define `rho'= J( `nitems', `nclass', 0.0)
	scalar `pvalue'=0.0
	
	forvalues i=1/`=`nclass'' {
		local value : word `i' of `null_gammalist'
		mat `gammaM'[1,`i']=`value'
	}
	
	forvalues i=1/`numList' {
		local value : word `i' of `null_rholist'
		local colIndex = mod(`i',`=`nclass'')
		if( `colIndex' == 0 ){
			local colIndex = `=`nclass''
		}
		local rowIndex = ceil((`i')/`=`nclass'')
		mat `rho'[`rowIndex',`colIndex']=`value'
	} 

    //mat list `gammaM'
	//mat list `rho'
	
    forvalues i=1/`num_bootstrap'{
		mata: simulateDataset("`gammaM'", "`rho'", `simulate_samplesize')
		ds
		qui describe
		//local nvars = `r(k)'
		//dis "nvars = " `nvars'
		unab varlist: _all 
		local count 1
		foreach var of local varlist {
		   if(`count' <= `nitems') {
			rename `var' item`count++'
			//local items "`items' `var'"
		   }
		   else {
			rename `var' freq
		   }
		}				  
		ds
		drop if freq == 0

		local items `r(varlist)'
		local exclude "freq"
		local items: list items - exclude
		//di "`items'"

        local categories 
		forvalues j=1/`=`nitems'' {
			local categories `categories' 2
		}
        //dis `categories'
		
		doLCA `items', ///
			  nstart(`null_nstarts') ///
			  nclass(`=`nclass'') ///
			  freq(freq) ///
			  seed(123456) ///
			  categories(`categories')  ///
			  core(`cores')  
	    local nullLikehood = r(loglikelihood)
		//dis "loop " `i' " nullLikehood" `nullLikehood'
		
		qui doLCA `items', ///
			  nstart(`alt_nstarts') ///
			  nclass(`=`nclass'+1') ///
			  freq(freq) ///
			  seed(123456) ///
			  categories(`categories')  ///
			  core(`cores')  
	    local altLikehood = r(loglikelihood)
		//dis "loop " `i' " altLikehood" `altLikehood'

		 //mat `loglikelihoodDiff'[`i',1]= abs(`nullLikehood'-`altLikehood')
		 scalar `loglikelihoodDiff' = abs(`nullLikehood'-`altLikehood')
		 scalar `bootstrapTestStat' = `loglikelihoodDiff' > abs(`null_loglikelihood'-`alt_loglikelihood')
		 scalar `pvalue' = `pvalue' + `bootstrapTestStat'
		 //mat list `loglikelihoodDiff'
	     cap drop _all
	}
	
    /*forvalues i=1/`num_bootstrap'{
		mat `bootstrapTestStat'[`i',1]=  `loglikelihoodDiff'[`i',1] > abs(`null_loglikelihood'-`alt_loglikelihood')
	}*/
	//mat list `bootstrapTestStat'
	//matsum `bootstrapTestStat', r(`p') d
	//mata : st_numscalar("`pvalue'", colsum(st_matrix("`bootstrapTestStat'")))
	ereturn scalar p_value = (1+`pvalue')/(1+`num_bootstrap')
		 
end 
